package imagefilters

/**
 * This class implements the various image filters
 */
object ImageFilters {

  def filter(src: Array[Array[Int]], func: (Int, Int, Int, Int, Int) => Int): Array[Array[Int]] = {
    val height: Int = src.length
    val width: Int = if (height == 0) 0 else src(0).length
    val dst: Array[Array[Int]] = Array.ofDim(height, width)
    for (y: Int <- 0 until height) {
      for (x: Int <- 0 until width) {
        dst(y)(x) = func(src(y)(x), x, y, width, height)
      }
    }
    return dst
  }
  def filter(src: Array[Array[Int]], func: (Int, Int, Int) => Int): Array[Array[Int]] = filter(src, (value, x, y, width, height) => func(value, x, y))
  def filter(src: Array[Array[Int]], func: (Int) => Int): Array[Array[Int]] = filter(src, (value, x, y, width, height) => func(value))

  def duplicate(a: Array[Array[Int]]): Array[Array[Int]] = filter(a, (value) => value)
  def threshold(a: Array[Array[Int]], thresh: Int): Array[Array[Int]] = filter(a, (value) => if (value > thresh) 255 else 0)
  def mean(a: Array[Array[Int]]): Array[Array[Int]] = filter(a, (value, x, y, width, height) => {
    if (x == 0 || x == width - 1 || y == 0 || y == height - 1) value
    else {
      var avg: Double = 0
      for (dy: Int <- -1 to 1) {
        for (dx: Int <- -1 to 1) {
          avg += a(y+dy)(x+dx)
        }
      }
      (avg/9.0).toInt
    }
  })
  def mean(a: Array[Array[Int]], radius: Int): Array[Array[Int]] = {
    if (radius < 0) throw new IllegalArgumentException("radius must be >= 0")
    val diameter: Int = 2 * radius + 1

    filter(a, (value, x, y, width, height) => {
      if (x <= radius - 1 || x >= width - radius || y <= radius - 1 || y >= height - radius) value
      else {
        var avg: Double = 0
        for (dy: Int <- -radius to radius) {
          for (dx: Int <- -radius to radius) {
            avg += a(y+dy)(x+dx)
          }
        }
        (avg/(diameter * diameter)).toInt
      }
    })
  }
}
